﻿# -*- coding: UTF-8 -*-

# shortLink: Bitly for NVDA
# http://dev.bitly.com/api.html

# Version: 2.0
# Date: 23/01/2013
# Now we use access-token instead of API key


import addonHandler
import globalPluginHandler
import urllib
import api
import wx
import gui
import textInfos
import scriptHandler
import languageHandler
import config
import os
import cPickle
import shutil
from cStringIO import StringIO
from configobj import ConfigObj
from validate import Validator
from logHandler import log

addonHandler.initTranslation()

config.initConfigPath(None)
savePath = os.path.join(os.path.dirname(__file__), "..", "settings")
iniFileName = os.path.join(savePath, "shortLink.ini")

confspec = ConfigObj(StringIO("""#Configuration file

[Bitly]
key = string(default="")
"""), encoding="UTF-8", list_values=False)
confspec.newlines = "\r\n"
conf = ConfigObj(iniFileName, configspec = confspec, indent_type = "\t", encoding="UTF-8")
val = Validator()
conf.validate(val)

accessKey = conf["Bitly"]["key"]

class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(globalPluginHandler.GlobalPlugin, self).__init__()
		self.menu = gui.mainFrame.sysTrayIcon.menu
		self.bitlyMenu = wx.Menu()
		self.mainItem = self.menu.AppendSubMenu(self.bitlyMenu, _("Short lin&k"), _("Short link menu (Bitly for NVDA)"))
		self.shortItem = self.bitlyMenu.Append(wx.ID_ANY, _("Sho&rten URL..."), _("Bitly: shorten URL"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onShort, self.shortItem)
		self.shareItem = self.bitlyMenu.Append(wx.ID_ANY, _("Sh&are link..."), _("Bitly: copy link to share"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onShare, self.shareItem)
		self.statsItem = self.bitlyMenu.Append(wx.ID_ANY, _("S&tats..."), _("Bitly: link stats"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onStats, self.statsItem)
		self.deleteItem = self.bitlyMenu.Append(wx.ID_ANY, _("De&lete link..."), _("Bitly: delete saved link"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onDelete, self.deleteItem)
		self.addItem = self.bitlyMenu.Append(wx.ID_ANY, _("Add li&nks from file..."), _("Bitly: add links from external file"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onAddLinks, self.addItem)
		self.saveItem = self.bitlyMenu.Append(wx.ID_ANY, _("Add link&s from Bitly..."), _("Bitly: add links from history"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onSaveFromBitly, self.saveItem)
		self.bitlyItem = self.bitlyMenu.Append(wx.ID_ANY, _("&Open Bitly site"), _("Bitly: home page"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onBitly, self.bitlyItem)
		self.settingsItem = self.bitlyMenu.Append(wx.ID_ANY, _("&Bitly settings..."), _("Bitly: access token"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onSettings, self.settingsItem)
		self.copyItem = self.bitlyMenu.Append(wx.ID_ANY, _("&Copy settings folder..."), _("Bitly: copy settings folder"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onCopy, self.copyItem)
		self.restoreItem = self.bitlyMenu.Append(wx.ID_ANY, _("R&estore settings folder..."), _("Bitly: restore settings"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onRestore, self.restoreItem)
		self.aboutItem = self.bitlyMenu.Append(wx.ID_ANY, _("Open info&rmation file"), _("Open documentation for current language"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onAbout, self.aboutItem, _("Open documentation for current language"))

		self.url = ""
		self.longUrl = ""
		self.pickle = os.path.join(savePath, "links.pickle")
		self.states = []
		self.titlesList = []
		self.linksList = []
		self.linksHistory = []


	def terminate(self):
		try:
			self.menu.RemoveItem(self.mainItem)
		except wx.PyDeadObjectError:
			pass


	def doLinksList(self):
		self.states = []
		self.linksList = []
		self.titlesList = []
		try:
			self.states = cPickle.load(file(self.pickle, "r"))
		except IOError:
			return
		for item in self.states:
			for k, v in item.iteritems():
				if k not in self.linksList:
					self.linksList.append(k)
					self.titlesList.append(v)


	def onStats(self, evt):
		self.doLinksList()
		if len(self.linksList) == 0:
			return
		choicesList = []
		for link, title in zip(self.linksList, self.titlesList):
			element = "{1}: {0}".format(link, title)
			choicesList.append(element)
		dlg = wx.SingleChoiceDialog(gui.mainFrame, _("Select a link for opening its stats page"), _("Stats for your selected link"), choices=choicesList)
		dlg.SetSelection(0)
		gui.mainFrame.prePopup()
		try:
			result = dlg.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			index = dlg.GetSelection()
			link = self.linksList[index]+"+"
			try:
				os.startfile(link)
			except WindowsError:
				pass


	def onDelete(self, evt):
		self.doLinksList()
		if len(self.linksList) == 0:
			return
		choicesList = []
		for link, title in zip(self.linksList, self.titlesList):
			element = "{1}: {0}".format(link, title)
			choicesList.append(element)
		dlg = wx.SingleChoiceDialog(gui.mainFrame, _("Select the link you wish to delete"), _("Delete a link"), choices=choicesList)
		dlg.SetSelection(0)
		gui.mainFrame.prePopup()
		try:
			result = dlg.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			index = dlg.GetSelection()
			link = self.linksList[index]
			try:
				expandedLink = self.expandUrl(link)
			except:
				raise
			if gui.messageBox(_("Are you sure you want to delete this link: %s?") % ": ".join([link, expandedLink]),
			_("Deleting a saved link!"),
			wx.YES|wx.NO|wx.ICON_WARNING)==wx.YES:
				self.states.pop(index)
				try:
					cPickle.dump(self.states, file(self.pickle, "wb"))
				except:
					log.debugWarning("Error saving new links", exc_info=True)



	def onShare(self, evt):
		self.doLinksList()
		if len(self.linksList) == 0:
			return
		choicesList = []
		for link, title in zip(self.linksList, self.titlesList):
			element = "{1}: {0}".format(link, title)
			choicesList.append(element)
		dlg = wx.SingleChoiceDialog(gui.mainFrame, _("Select the link you want to share"), _("Copy your selected link"), choices=choicesList)
		dlg.SetSelection(0)
		gui.mainFrame.prePopup()
		try:
			result = dlg.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			index = dlg.GetSelection()
			link = self.linksList[index]
			try:
				expandedLink = self.expandUrl(link)
			except:
				raise
			if gui.messageBox(_("Are you sure you want to copy this link: %s?") % ": ".join([link, expandedLink]),
			_("Copying a saved link!"),
			wx.YES|wx.NO|wx.ICON_WARNING)==wx.YES:
				api.copyToClip(unicode(link))


	def onSaveFromBitly(self, evt):
		self.getUserLinksHistory()
		if len(self.linksHistory) == 0:
			return
		dlg = wx.SingleChoiceDialog(gui.mainFrame, _("Select the link you want to save"), _("Save your selected link"), choices=self.linksHistory)
		dlg.SetSelection(0)
		gui.mainFrame.prePopup()
		try:
			result = dlg.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			index = dlg.GetSelection()
			self.url = self.linksHistory[index].split(": ")[1]
			self.titleDialog()


	def shortenUrl(self, URLToShorten):
		try:
			longUrl = urllib.urlencode(dict(longUrl=URLToShorten))
			global accessKey
			accessToken = urllib.urlencode(dict(access_token=accessKey))
			encodedurl="https://api-ssl.bitly.com/v3/shorten?%s&%s" % (accessToken, longUrl)
			request = urllib.urlopen(encodedurl)
			response = request.read()
			request.close()
			response_dict = eval(response)
			short_url = response_dict["data"]["url"].replace("\/", "/")
			return short_url
		except:
			raise


	def expandUrl(self, bitlyLink):
		try:
			short_url = urllib.urlencode(dict(shortUrl=bitlyLink))
			global accessKey
			accessToken = urllib.urlencode(dict(access_token=accessKey))
			encodedurl="https://api-ssl.bitly.com/v3/expand?%s&%s" % (accessToken, short_url)
			request = urllib.urlopen(encodedurl)
			response = request.read()
			request.close()
			response_dict = eval(response)
			response_dict_data = response_dict["data"]
			expandedUrl = response_dict_data["expand"][0]["long_url"].replace("\/", "/")
			return expandedUrl
		except:
			raise


	def getUserLinksHistory(self):
		try:
			global accessKey
			accessToken = urllib.urlencode(dict(access_token=accessKey))
			lim = urllib.urlencode(dict(limit=100))
			encodedurl="https://api-ssl.bitly.com/v3/user/link_history?%s&%s" % (accessToken, lim)
			request = urllib.urlopen(encodedurl)
			response = request.read()
			request.close()
			false = False
			true = True
			null = ""
			response_dict = eval(response)
			response_dict_data = response_dict["data"]
			dataList = response_dict_data["link_history"]
			linksHistoryList = []
			for i, item in enumerate(dataList):
				currentLink = "%d. %s: %s" % (i+1, item["long_url"], item["link"])
				linksHistoryList.append(currentLink)
			self.linksHistory = linksHistoryList
		except:
			raise


	def onShort(self, evt):
		scriptHandler.executeScript(self.script_shortLink, None)


	def onBitly(self, evt):
		try:
			os.startfile("http://www.bitly.com")
		except WindowsError:
			pass


	def onSettings(self, evt):
		global accessKey
		d = wx.TextEntryDialog(gui.mainFrame, _("Your access token"), _("Bitly: access token"), defaultValue=accessKey)
		gui.mainFrame.prePopup()
		try:
			result = d.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			accessKey = d.GetValue()
			conf["Bitly"]["key"] = accessKey
			try:
				conf.validate(val, copy=True)
				conf.write()
				log.info("ShortLink add-on configuration saved")
			except Exception, e:
				log.warning("Could not save add-on configuration")
				log.debugWarning("", exc_info=True)
				raise e


	def onAddLinks(self, evt):
		d =wx.FileDialog(gui.mainFrame, _("Choose a file containing Bitly links"), savePath, "addLinks.pickle", _("Pickle files (*.pickle) |*.pickle"), wx.FD_OPEN)
		gui.mainFrame.prePopup()
		try:
			result = d.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			try:
				newLinks = cPickle.load(file(d.GetPath(), "r"))
				newLinks.reverse()
			except IOError:
				return
			self.doLinksList()
			for item in newLinks:
				for k, v in item.iteritems():
					if k in self.linksList:
						continue
					else:
						self.states.insert(0, item)
			try:
				cPickle.dump(self.states, file(self.pickle, "wb"))
			except:
				log.debugWarning("Error saving new links", exc_info=True)



	def onCopy(self, evt):
		configPath = config.getUserDefaultConfigPath()
		dlg = wx.DirDialog(gui.mainFrame, _("Select a folder for copying your Bitly settings"), configPath, wx.DD_DEFAULT_STYLE)
		gui.mainFrame.prePopup()
		result = dlg.ShowModal()
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			copyPath = os.path.join(dlg.GetPath(), "shortLinkSettings")
			try:
				shutil.rmtree(copyPath, ignore_errors=True)
				shutil.copytree(savePath, copyPath)
			except WindowsError:
				wx.CallAfter(gui.messageBox,_("Folder not copied"), _("Copy Error"), wx.OK|wx.ICON_ERROR)


	def onRestore(self, evt):
		configPath = config.getUserDefaultConfigPath()
		linksPath = os.path.join(configPath, "shortLinkSettings")
		dlg = wx.DirDialog(gui.mainFrame, _("Select the settings folder you wish to restore"), linksPath, wx.DD_DIR_MUST_EXIST | wx.DD_DEFAULT_STYLE)
		gui.mainFrame.prePopup()
		result = dlg.ShowModal()
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			linksPath = dlg.GetPath()
			try:
				shutil.rmtree(savePath, ignore_errors=True)
				shutil.copytree(linksPath, savePath)
			except WindowsError:
				wx.CallAfter(gui.messageBox,_("Folder not copied"), _("Copy Error"), wx.OK|wx.ICON_ERROR)

	def getDocFolder(self):
		langs = [languageHandler.getLanguage(), "en"]
		for lang in langs:
			docFolder = os.path.join(os.path.dirname(__file__), "..", "doc", lang)
			if os.path.isdir(docFolder):
				return docFolder
			if "_" in lang:
				tryLang = lang.split("_")[0]
				docFolder = os.path.join(os.path.dirname(__file__), "..", "doc", tryLang)
				if os.path.isdir(docFolder):
					return docFolder
				if tryLang == "en":
					break
			if lang == "en":
				break
		return None


	def getDocPath(self, docFileName):
		docPath = self.getDocFolder()
		if docPath is not None:
			docFile = os.path.join(docPath, docFileName)
			if os.path.isfile(docFile):
				docPath = docFile
		return docPath


	def onAbout(self, evt):
		try:
			os.startfile(self.getDocPath("readme.html"))
		except WindowsError:
			pass


	def titleDialog(self):
		self.doLinksList()
		defaultTitle = ""
		for link, title in zip(self.linksList, self.titlesList):
			if self.url == link:
				defaultTitle = title
				break
		d = wx.TextEntryDialog(gui.mainFrame, _("Title for your saved link"), _("Bitly: Name for Bitly link"), defaultValue=defaultTitle)
		gui.mainFrame.prePopup()
		try:
			result = d.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK and d.GetValue():
			self.doLinksList()
			title = d.GetValue().encode("mbcs")
			item = {self.url: title}
			if self.url in self.linksList:
				i = self.linksList.index(self.url)
				self.states[i][self.url] = title
			else:
				self.states.insert(0, item)
			try:
				cPickle.dump(self.states, file(self.pickle, "wb"))
			except:
				log.debugWarning("Error saving link", exc_info=True)



	def setAddressDialog(self):
		obj=api.getFocusObject()
		treeInterceptor=obj.treeInterceptor
		if hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough:
			obj=treeInterceptor
		try:
			info=obj.makeTextInfo(textInfos.POSITION_SELECTION)
		except (RuntimeError, NotImplementedError):
			info=None
		if not info or info.isCollapsed:
			textUrl = self.longUrl
		else:
			textUrl = info.text
		d = wx.TextEntryDialog(gui.mainFrame, _("The URL you wish to shorten"), _("Web address"), defaultValue=textUrl)
		def callback(result):
			if result == wx.ID_OK:
				# Make sure this happens after focus returns to the document.
				wx.CallLater(100, self.doSetAddress, d.GetValue())
		gui.runScriptModalDialog(d, callback)

	def doSetAddress(self, text):
		if not text:
			return
		self.longUrl = text
		self.url = self.shortenUrl(self.longUrl)
		if self.url and api.copyToClip(unicode(self.url)):
			self.titleDialog()


	def script_shortLink(self, gesture):
		self.setAddressDialog()

	script_shortLink.__doc__ = _("Opens a dialog for setting the URL you wish to be shortened")


	__gestures = {
		"kb:control+shift+u": "shortLink",
	}
